#
# AllWorkflows Script
#
# Version: 4.4
# 
# This script was written as a starting off point for performing logic to all Nintex Workflows
# in your farm environment.
# I do not claim that this is 100% perfect and you should be careful using it in a production environment.
#
# USAGE
# =====
# Run the script to Find Workflows with an Action:
#   .\AllWorkflowsV4.ps1 -nwoperation FindWorkflowsWithAction -nwlogin domain\username -nwpassword password -nwaction "actionname"
# eg. to find all the workflows that contain the Math operation action
#   .\AllWorkflowsV4.ps1 -nwoperation FindWorkflowsWithAction -nwlogin domain\username -nwpassword password -nwaction "Math operation"
# Run the script to Export:
#   .\AllWorkflowsV4.ps1 -nwoperation ExportAll -nwlogin domain\username -nwpassword password
# Run the script to Republish:
#   .\AllWorkflowsV4.ps1 -nwoperation RepublishAll -nwlogin domain\username -nwpassword password
# Run the script to Find all Workflows with Verbose Logging turned on
#   .\AllWorkflowsV4.ps1 -nwoperation FindVerboseLogging -nwlogin domain\username -nwpassword password
#
# History
# =======
# v4.4 - updated thes script to find all workflows with Verbose Logging Enabled
# v4.3 - updated thes script to handle parameters better
#        It now support this format
#        -paramname paramvalue
#        nwoperation
#        nwlogin
#        nwpassword
#        nwtimeout
#        nwaction
# v4.2 - added support for non-list workflows (site, reusable, sitecollection reusable)
#        add hashtable to store already made web service connections (don't need to connect again)
#

Param(
    [parameter(Mandatory=$true)]
    [alias("nwoperation")]
    $operation,
    [parameter(Mandatory=$true)]
    [alias("nwlogin")]
    $login,
    [alias("nwpassword")]
    $password,
    [alias("nwtimeout")]
    $timeout = 100000,
    [alias("nwaction")]
    $actionname)

[IO.Directory]::SetCurrentDirectory((Convert-Path (Get-Location -PSProvider FileSystem)))

# accept SSL Certificates if this is an SSL call
[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {$true}

$siteHashTable = @{};

# build the credential object
$secpasswd = ConvertTo-SecureString $password -AsPlainText -Force
$credential = New-Object System.Management.Automation.PSCredential($login,$secpasswd);

# check if we are in the same location as the nwadmin.exe
if(Test-Path(".\nwadmin.exe"))
{ 
  # find all the workflows and store them in a variable
  $foundworkflows = .\nwadmin -o FindWorkflows

  # read the contents of the NWAdmin -o FindWorkflows
  # $foundworkflows = get-content "C:\Program Files\Nintex\Nintex Workflow 2010\workflows.txt"

  foreach($line in $foundworkflows)
  {
    if($line.StartsWith("Active at "))
    {
      # get the site url
      $site = $line.Replace("Active at ","");
    }
    if($line.StartsWith("-- "))
    { 
      # get the list name
      $list = $line.Replace("-- ","");
    }
    if($line.StartsWith("---- "))
    {
      try
      {
        # get the workflow name
        $workflowname = $line.Replace("---- ","");

        $webserviceurl = $site + "/_vti_bin/NintexWorkflow/Workflow.asmx"
        $knownSite = $siteHashTable.ContainsKey($webserviceurl);
        if($knownSite -eq $true)
        {
          # echo 'This is a known site';
          $page = $siteHashTable.Get_Item($webserviceurl);
        }
        else
        {
          # get web service proxy
          $page = New-WebServiceProxy -Uri $webserviceurl -Credential $credential;
          $page.Url = $webserviceurl;
          $page.Timeout = $timeout;
          # ($page | Get-Member -Name ExportWorkflow).definition;
  
          # echo 'Adding site to hash table';
          $siteHashTable.Add($webserviceurl,$page);
        }

        # export workflow
        if($list -eq "Site Workflow")
        {$exportedworkflow = $page.ExportWorkflow($workflowname,$list,'Site');}
        elseif($list -eq "Reusable workflow template")
        {$exportedworkflow = $page.ExportWorkflow($workflowname,$list,'Reusable');}
        elseif($list -eq "Site collection reusable workflow template")
        {$exportedworkflow = $page.ExportWorkflow($workflowname,$list,'GloballyReusable');}
        else
        {$exportedworkflow = $page.ExportWorkflow($workflowname,$list,'List');}

        switch($operation)
        {
          "RepublishAll"
          {
            # republish this workflow
            if($list -eq "Site Workflow")
            {$publishresult = $page.PublishFromNWFXml($exportedworkflow,"",$workflowname,$True);}
            elseif($list -eq "Reusable workflow template")
            {$publishresult = $page.PublishFromNWFXml($exportedworkflow,"",$workflowname,$True);}
            elseif($list -eq "Site collection reusable workflow template")
            {$publishresult = $page.PublishFromNWFXml($exportedworkflow,"",$workflowname,$True);}
            else
            {$publishresult = $page.PublishFromNWFXml($exportedworkflow,$list,$workflowname,$True);}              
          }
          "ExportAll"
          {
            $localfile = "{0}\{1}.nwf" -f [IO.Directory]::GetCurrentDirectory(),$workflowname;
            # $message = "Exporting workflow to - {0}" -f $localfile;
            # echo $message;
            $stream = [System.IO.StreamWriter] $localfile;
            $stream.WriteLine($exportedworkflow);
            $stream.close();
          }
          "FindActionsUsed"
          {
            $actions = [regex]::matches($exportedworkflow, "(?<=TLabel&gt;)[\w ]+(?=&lt;/TLabel)");
            $arrayUniqueActions = @();

            # iterate through each action found
            foreach($action in $actions)
            {
              $arrayUniqueActions = $arrayUniqueActions + $action.Value;
            }
            $arrayUniqueActions = $arrayUniqueActions | select -uniq;
            $arrayUniqueActions;
          }
          "FindWorkflowsWithAction"
          {
            $actions = [regex]::matches($exportedworkflow, "(?<=TLabel&gt;)[\w ]+(?=&lt;/TLabel)");

            # iterate through each action found
            foreach($action in $actions)
            {
              if($action.Value -eq $actionName)
              {
                $message = "{0} - {1} - {2}" -f $site,$list,$workflowname;
                echo $message;
                break;
              }
            }
          }
          "FindVerboseLogging"
          {
            $verboselogging = [regex]::match($exportedworkflow, "(?<=Parameter Name=\`"VerboseLogging\`"&gt;&lt;PrimitiveValue Value=\`")[\w ]+(?=\`" ValueType=\`"Text\`")");
            if($verboselogging.Value -eq "true")
            {
              $message = "Verbose logging enabled on : ";
              echo $message;
              $message = "  Site - {0}" -f $webserviceurl;
              echo $message;
              $message = "  List - {0}" -f $list
              echo $message;
              $message = "  Workflow - {0}" -f $workflowname
              echo $message;
             }
          }
          default
          {
            echo "No operation";
            break;
          }
        }
      }
      catch
      {
        $message = "Site - {0}" -f $webserviceurl;
        echo $message;
        $message = "List - {0}" -f $list
        echo $message;
        $message = "Workflow - {0}" -f $workflowname
        echo $message;
        # if ($_.Exception.InnerException) {
        #   echo $_.Exception.InnerException.message;
        # }
        # Write-Error ("Error - " + $_);
        # $error[0];
        # echo $_.Exception.Message;
        # $_.Exception;
    # $_.Exception.Message[0];
        $exportedworkflow = "";

        # end the loop (end the script) when an exception occurs
        break;
      }

      # only do this for the first workflow
      # break;
    }
  }
}
else
{
  echo "NWAdmin doesn't exist.  Change directory to where NWAdmin.exe lives.";
}